#pragma once
#include "intalk.h"
#include "common.h"
#include "INpSession.h"
#include "INpPacket.h"
#include "NtalkModel.h"

class CNtalkClientSession :
	public INTalkClient, 
	public _INpSessionEvents
{
public:
	CNtalkClientSession(INTalkClientEvents* pEvents);
	virtual ~CNtalkClientSession(void);

	virtual BOOL	CreateChatSession(LPCTSTR strSrvAddress, int nPort, 
										ULONG nRoomID, LPCTSTR pszCreaterID, 
										LPCTSTR pszUserName, LPCTSTR pszNickName, BOOL bCreation = FALSE);

	virtual BOOL RequestReCoverSession();

	BOOL			WSASendErrorLogging( WSABUF wsaBuf);

	void			DebugStringWSAError();

	virtual void	DestroyChatSession(); 
	virtual BOOL	RequestInitRoom();
	virtual BOOL	RegisterChatUsers(std::vector<PINVITE_USER> vecUserList);
	virtual BOOL	RequestSendChatMsg(LPCTSTR pszMsg, CHARFORMAT2* pFont, UINT nTid, UINT nMsgSeq, BYTE MsgType);
	virtual BOOL	RequestTyping(NTalkTyping state);
	virtual BOOL	RequestChatJoin();
	virtual BOOL	RequestForceChatReJoin();
	virtual BOOL	RequestChatLeave();
	virtual BOOL	RequestForceLeave( LPCTSTR pszUserID, LPCTSTR pszUserName, LPCTSTR pszNickName );
	virtual BOOL	RequestAckRecvMsg(UINT nMsgSeq, UINT nTid, INT& nReason);
	virtual BOOL	RequestAckMsgResult(UINT nMsgSeq, UINT nTid, UINT nResultSeq);
	virtual BOOL	RequestSendPing();
	virtual BOOL	RequestFileSend(std::vector<std::wstring> vecReceiverIds, LPCTSTR pszFileName, UINT nFileSize, LPCTSTR pszURL, UINT nTid, UINT nMsgSeq);
	virtual BOOL	RequestAckRecvFile(UINT nMsgSeq, UINT nTid);

	virtual void	SetNickname(LPCTSTR pszNickName);
	virtual int		GetNetworkError();
	virtual void	SetNetworkError(int error);
	
	void			ServerSessionEnd();

	virtual void CALLBACK OnNpSession_Connect(INpSession *pSession, int nSocketError);
	virtual void CALLBACK OnNpSession_Close(INpSession *pSession, int nSocketError, bool bRecvInvalidPacket);
	virtual void CALLBACK OnNpSession_RecvPacket(INpSession *pSession, INpPacket * pPacket);
	virtual void CALLBACK OnNpSession_SendReady(INpSession *pSession);

private:
	// wchar memcpy
	int MemcpyWideCharsWithSize(char * pBuffer,LPCWSTR pWString);
	int				GetCharFormat2Len(CHARFORMAT2* pFont);

	void RecvPushRecvMsg(INpPacket * pPacket);		//ȭޱ

	BOOL Read4Byte(int &nTotalRcv, void *result, const char** iterator);
	BOOL Read2Byte(int &nTotalRcv, void *result, const char** iterator);
	BOOL Read1Byte(int &nTotalRcv, void *result, const char** iterator);
	BOOL ReadString(int& nTotalRcv, char** ppString, const char** iterator, int length);
	BOOL ReadSendUserID(int &nTotalRcv, RES_RECVMSG &stResRecvMsg, const char** iterator);
	BOOL ReadSendUserID(int &nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator);
	BOOL ReadRoomID(int &nTotalRcv, void *nRoomID, const char** iterator);
	BOOL ReadSenderNickName(int &nTotalRcv, RES_RECVMSG &stResRecvMsg, const char** iterator);
	BOOL ReadSenderNickName(int &nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator);
	BOOL ReadOSType( int& nTotalRcv, RES_RECVMSG &stResRecvMsg, const char** iterator );
	BOOL ReadOSType( int& nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator );
	BOOL ReadTime( int& nTotalRcv, RES_RECVMSG &stResRecvMsg, const char** iterator );
	BOOL ReadTime( int& nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator );
	BOOL ReadMessage( int& nTotalRcv, RES_RECVMSG &stResRecvMsg, const char** iterator );
	BOOL ReadMessageSeq( int& nTotalRcv, RES_RECVMSG &stResRecvMsg, const char** iterator );
	BOOL ReadMessageSeq( int& nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator );
	BOOL ReadTid( int& nTotalRcv, RES_RECVMSG &stResRecvMsg, const char** iterator );
	BOOL ReadTid( int& nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator );
	BOOL ReadMsgType( int& nTotalRcv, RES_RECVMSG &stResRecvMsg, const char** iterator );
	BOOL ReadMsgType( int& nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator );
	BOOL ReadFonts( int& nTotalRcv, RES_RECVMSG &stResRecvMsg, const char** iterator, CHARFORMAT2** pCharFormat);
	BOOL ReadFileName(int &nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator);
	BOOL ReadFileSize(int& nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator);
	BOOL ReadFileURL(int &nTotalRcv, RES_RECVFILE &stResRecvFile, const char** iterator);

	void RecvPushTyping(INpPacket * pPacket);			//  Է ˸
	void RecvResJoin(INpPacket * pPacket);				//ȭû 
	void RecvResForceReJoin(INpPacket * pPacket);		//ȭ氭 
	void RecvResForceLeave(INpPacket * pPacket);		//û 
	void RecvPushForceLeave(INpPacket * pPacket);		// ˸
	void RecvPushSessionClose(INpPacket * pPacket);		//ǲ ˸
	void RecvPushNewJoiner(INpPacket * pPacket);		//ȭ  ˸
	void RecvPushForceReJoiner(INpPacket * pPacket);	//ȭ   ˸
	void RecvPushLeaveJoiner(INpPacket * pPacket);		//ȭ  ˸
	void RecvAckSendMsg(INpPacket * pPacket);			//ȭ ۿ   
	void RecvResInitRoom(INpPacket * pPacket);			//ȭ  
	void RecvResInviteList(INpPacket * pPacket);		//ȭ  
	void RecvPushRoomEnd(INpPacket * pPacket);			//ȭ ˸
    void RecvPushSessionEnd(INpPacket * pPacket);       //ȭ   ˸
	void RecvServerNotReady();							//äü غ߻
	void RecvPushMsgFail(INpPacket * pPacket);			//ȭ޽о˸
	void RecvPushMsgResult(INpPacket * pPacket);		//ȭ۰˸
	void RecvAckSendFile(INpPacket * pPacket);			// ۿ   
	void RecvPushRecvFile(INpPacket * pPacket);			//  ˸
	void RecvPong();

	int EncryptMsg(IN LPCTSTR strID, IN char* szPlainMsg, OUT unsigned char* szEncryptedMsg);
	void DecryptMsg(IN LPCTSTR strID, IN unsigned char* szEncMsg, IN int nEncSize, OUT char* szPlainMsg);

	void ReleaseSession();
private:
	INpSession* m_pNpSession;
	int			m_networkError;
	CNtalkModel* m_pNtalkModel;
	INTalkClientEvents* m_pEvents;
};
